/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"


// A more complicated connection example which can specify additional 
// access parameters, such as read-only access, user privleges, etc.  
// NOTE:  Security must be
// enabled for a database before users may be specified; for details,
// see section 5 of the Matisse Server Administration Guide.

void printMsg(MtDatabase* db, NSString *msg)
{
    printf("%s %s\n",
	   [[db description] UTF8String],
	   [msg UTF8String]);
}

BOOL isReadOnly(MtDatabase* db)
{
  // see enum MtDataAccessMode in matisseCtx.h
  return ([db option:MT_DATA_ACCESS_MODE] == MT_DATA_READONLY);
}

void printState(MtDatabase* db)
{
  if (! [db isConnectionOpen]) {
    printMsg(db, @"not connected");
  } else {
    if ([db isTransactionInProgress]) {
      printMsg(db, @"read-write transaction underway");
    } else if ([db isVersionAccessInProgress]) {
      printMsg(db, @"read-only version access underway");
    } else {
      printMsg(db, @"no transaction underway");
    }
  }

  if (isReadOnly(db)) {
    printMsg(db, @"data access is read-only");
  } else {
    printMsg(db, @"data access is read-write");
  }
}


int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname, *username = nil, *passwd = nil;
  int accessMode = MT_DATA_MODIFICATION;

  if (argc < 4) {
    printf("Usage: %s <host> <db> [-r|-w|-d]\n", argv[0]);
    return -1;
  }
  if (strcmp(argv[3], "-w") == 0) {
    accessMode = MT_DATA_MODIFICATION;
  }
  if (strcmp(argv[3], "-r") == 0) {
    accessMode = MT_DATA_READONLY;
  }
  if (strcmp(argv[3], "-d") == 0) {
    accessMode = MT_DATA_DEFINITION;
  }

#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname];

    [db setOption:MT_DATA_ACCESS_MODE :accessMode];

    [db open:username :passwd];

    if (isReadOnly(db)) {
      [db startVersionAccess];
    } else {
      [db startTransaction];
    }

    printState(db);

    if (isReadOnly(db)) {
      [db endVersionAccess];
    } else {
      [db rollback];
    }

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
