/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"

// A more complete example using classes to modify Matisse objects.
// Uses an image file to stream data in/out of an attribute. Also shows
// how to catch a specific exception.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  @try {
    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chaps_4_5"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init: @"" : @"examples.objc_examples.chaps_4_5"];
    db = [(MtDatabase *)[MtDatabase alloc] init : hostname : dbname andFactory: factory];

    [db open];

    [db startTransaction];

    // create a new Employee
    Employee* e = [[Employee alloc] init: db];
    // setting string attributes
    [e setComment: @"setting values"];
    [e setFirstName: @"John"];
    [e setLastName: @"Jones"];

    // setting numbers
    [e setAge: 42];

    // setting Date
    NSDate *ts = [NSDate dateWithString : @"2012-05-06 00:00:00 +0000" ];
    [e setHireDate : ts];
 
    // setting Numeric
    NSDecimalNumber* num = [NSDecimalNumber decimalNumberWithString : @"2000.50"];
    [e setSalary: num];

    // getting values
    printf("getting values\n");
    printf("%s\n", [[e comment] UTF8String]);
    printf("Employee: %s %s\n", 
	   [[e firstName] UTF8String],
	   [[e lastName] UTF8String]);

    // suppress output if no value set
    // use generated isAgeNull method to check if value is null
    printf("Age: ");
    if (! [e isAgeNull]) {
      printf(" %d years old\n", [e age]);
    } else {
      printf("age not set\n");
    }
    printf("Number of Dependents: %d (default value: %s)\n", 
	   [e dependents],
	   [e isDependentsDefaultValue] ? "Yes" : "No");

    printf("Salary: %s\n", [[[e salary] description] UTF8String]);
    printf("Hired on: %s\n", [[[e hireDate] description] UTF8String]);


    // changing values (getting and setting)
    [e setDependents: ([e dependents] + 2)];
    [e setSalary: [[e salary] decimalNumberByMultiplyingBy: [NSDecimalNumber decimalNumberWithString : @"1.10"]]];
    [e setComment: @"changing values"];

    // removing a value
    [e removeAge];

    // Getting again to show effect of removing value
    // similar to above
    printf("\n%s\n", [[e comment] UTF8String]);
    printf("Employee: %s %s\n", 
	   [[e firstName] UTF8String],
	   [[e lastName] UTF8String]);

    // suppress output if no value set
    // use generated isAgeNull method to check if value is null
    printf("Age: ");
    if (! [e isAgeNull]) {
      printf(" %d years old\n", [e age]);
    } else {
      printf("age not set\n");
    }
    printf("Salary: %s\n", [[[e salary] description] UTF8String]);

    // changing values with alternative methods (getting and setting)
    printf("getting and setting values with dot(.) and KVC\n");
    printf("comment: %s\n", [e.comment UTF8String]);
    e.comment = @"new value via dot";
    printf("comment: %s\n", [e.comment UTF8String]);
    [e setValue:@"even newer value with KVC methods" forKey:@"comment"]; 
    printf("comment: %s\n", [[e valueForKey:@"comment"] UTF8String]);


    // Blob access:
    // The Photo attribute's type is MtImage which is
    // streamable. Use an array of bytes to read an image file from
    // disk and store into the attribute.

    printf("1 - Image (Blob) access:\n");
    NSError *error;
    NSData* val = [NSData dataWithContentsOfFile:@"smiley.gif" options:0 error: &error];
    if (!val) {
      NSLog(@"unable to open smiley.gif file: %@\n", error);
    }
    [e setPhoto : val];
    printf("set photo from smiley.gif\n");


    NSData* valimage = [e photo];
    if (! [valimage writeToFile:@"out.gif" options:0 error: &error]) {
      NSLog(@"unable to create out.gif file: %@\n", error);
    }
    printf("save photo out.gif\n");

    printf("2 - Image (Blob) streaming:\n");
      
    Person *p = [[Person alloc] init: db];
    // setting string attributes
    [p setComment: @"setting image"];
    [p setFirstName: @"Bill"];
    [p setLastName: @"Williamson"];

    [p setPhotoElements: valimage : 0 : MT_BEGIN_OFFSET : YES];

    [p setPhotoElements: valimage : [valimage length] : MT_CURRENT_OFFSET : YES];

    MtSize numelts = [p photoSize];

    NSMutableData* valimagebuffer = [NSMutableData dataWithLength : numelts];
    numelts = [p photoElements: valimagebuffer : 0 : MT_BEGIN_OFFSET];

    numelts = [p photoElements: valimagebuffer : (unsigned int)[valimagebuffer length] : MT_CURRENT_OFFSET];
    if (! [valimagebuffer writeToFile:@"out2.gif" options:0 error: &error]) {
      NSLog(@"unable to create out2.gif file: %@\n", error);
    }
    printf("save buffer of %d elements into out2.gif\n", numelts);

    printf("3 - Image (Blob) streaming:\n");

    Person *p3 = [[Person alloc] init: db];
    // setting string attributes
    [p3 setComment: @"setting image"];
    [p3 setFirstName: @"Mary"];
    [p3 setLastName: @"Williamson"];

    NSFileHandle *fdr;
    fdr = [NSFileHandle fileHandleForReadingAtPath: @"smiley.gif"];
    if (!fdr) {
      NSLog(@"unable to open smiley.gif file\n");
    }
    // small buffer size for demo purpose
    MtSize maxbufsiz = 1024;

    [p3 setPhotoElements: nil : 0 : MT_BEGIN_OFFSET : YES];
    NSData* databuffer = [fdr readDataOfLength: maxbufsiz];
    while ([databuffer length] > 0) {
      [p3 setPhotoElements: databuffer : (unsigned int)[databuffer length] : MT_CURRENT_OFFSET : YES];
      databuffer = [fdr readDataOfLength: maxbufsiz];
      if (!databuffer) {
	NSLog(@"unable to read smiley.gif file\n");
      }
    }
    [fdr closeFile];

    NSString* filename = @"out3.gif";
    NSFileManager *filemgr;
    filemgr = [NSFileManager defaultManager];
    if ([filemgr fileExistsAtPath: filename ] == YES) {
      if ([filemgr removeItemAtPath: filename error: NULL]  == YES) {
        NSLog (@"Removed %s file", [filename UTF8String]);
      }
    }
    [filemgr createFileAtPath: filename contents: nil attributes: nil];

    // small buffer size for demo purpose
    MtSize bufsiz = 1024;
    MtSize imgsiz = 0;

    NSMutableData* valimagebuffer3 = [NSMutableData dataWithLength : bufsiz];

    NSFileHandle *file;
    file = [NSFileHandle fileHandleForWritingAtPath: filename];

    [file seekToFileOffset: 0];
    numelts = [p photoElements: valimagebuffer3 : 0 : MT_BEGIN_OFFSET];
    do {
      numelts = [p photoElements: valimagebuffer3 : bufsiz : MT_CURRENT_OFFSET];
      if (numelts < bufsiz) [valimagebuffer3 setLength : numelts];
      [file writeData: valimagebuffer3];
      imgsiz += numelts;
    } while (numelts == bufsiz);

    [file closeFile];
    printf("save buffers for a total of %d elements into out3.gif\n", imgsiz);

    [db commit];

    [db startVersionAccess];

    printf("\nAttempting to modify %s %s salary within read-only access\n", 
	   [[e firstName] UTF8String],
	   [[e lastName] UTF8String]);

    @try {
      [e setSalary: [[e salary] decimalNumberByMultiplyingBy: [NSDecimalNumber decimalNumberWithString : @"1.10"]]];
    }
    @catch (MtException *mte) {
      printf("ERROR: %s: (code=%d) %s\n", 
	     [[mte name] UTF8String], 
	     [mte errorCode], 
	     [[mte message] UTF8String] );
      if ([mte errorCode] == MATISSE_NOTRANS) {
	printf(" as expected...\n");
      } else {
	printf(" but was not expected...\n");
      }
    }

    [db endVersionAccess];

    [db startTransaction];
    printf("\nRemoving created object...\n");
    [[Person getClass:db] removeAllInstances];

    printf("%d Person(s) in the database %s\n", [Person instanceNumber:db],
	   [[db description] UTF8String]);

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  }
#endif

  printf("Done\n");

  return 0;
}
